@@ -25,7 +25,7 @@
 #include "Lexer.h"
 #include "PDFDoc.h"
 #include "OptionalContent.h"
-#include "Form.h"
+#include "AcroForm.h"
 #include "BuiltinFontTables.h"
 #include "FontEncodingTables.h"
 #include "Annot.h"
@@ -845,7 +845,7 @@ GBool Annot::setFillColor(Object *colorObj) {
   if (!colorObj->isArray()) {
     return gFalse;
   }
-  for (i = 0; i < colorObj->arrayGetLength(); ++i) {
+  for (i = 0; i < colorObj->arrayGetLength() && i < 4; ++i) {
     if (colorObj->arrayGet(i, &obj)->isNum()) {
       color[i] = obj.getNum();
     } else {
@@ -1287,11 +1287,11 @@ void Annot::drawText(GString *text, GString *da, int quadding, double margin,
     c = text2->getChar(i) & 0xff;
     if (c == '(' || c == ')' || c == '\\') {
       appearBuf->append('\\');
-      appearBuf->append(c);
+      appearBuf->append((char)c);
     } else if (c < 0x20 || c >= 0x80) {
       appearBuf->appendf("\\{0:03o}", c);
     } else {
-      appearBuf->append(c);
+      appearBuf->append((char)c);
     }
   }
   appearBuf->append(") Tj\n");
