@@ -679,11 +679,31 @@ void printpage(void)
 
                 if (ch >= SPACE || allchar) {
 		  if (utf8 && (ch & IS_UNICODE)) {
-		    mbch = UCStoUTF8(ch & MAX_UNICODE);
-		    if (BYTE1(mbch) != 0) putc((unsigned char)BYTE1(mbch), output);
-		    if (BYTE2(mbch) != 0) putc((unsigned char)BYTE2(mbch), output);
-		    if (BYTE3(mbch) != 0) putc((unsigned char)BYTE3(mbch), output);
-		    /* always */          putc((unsigned char)BYTE4(mbch), output);
+#ifdef WIN32
+		    wchar_t wch;
+		    HANDLE hStdout;
+		    DWORD ret;
+		    const int fd = fileno(output);
+
+		    if ((fd == fileno(stdout) || fd == fileno(stderr)) && _isatty(fd)) {
+		      if (fd == fileno(stdout))
+			hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
+		      else
+			hStdout = GetStdHandle(STD_ERROR_HANDLE);
+
+		      wch=(wchar_t)(ch & MAX_UNICODE); /* do not expect over BMP */
+		      WriteConsoleW(hStdout, &wch, 1, &ret, NULL);
+
+		    } else {
+#endif
+		      mbch = UCStoUTF8(ch & MAX_UNICODE);
+		      if (BYTE1(mbch) != 0) putc((unsigned char)BYTE1(mbch), output);
+		      if (BYTE2(mbch) != 0) putc((unsigned char)BYTE2(mbch), output);
+		      if (BYTE3(mbch) != 0) putc((unsigned char)BYTE3(mbch), output);
+		      /* always */          putc((unsigned char)BYTE4(mbch), output);
+#ifdef WIN32
+		    }
+#endif
 		  }
 		  else if (japan) {
 		    for (k = 0; k < kanji1; k++) {
@@ -833,11 +853,7 @@ void ruleaux(long rulewt, long ruleht, char ch)
 long horizontalmove(long amount)
 {
 
-#if defined(MSDOS) || defined(THINK_C)
-    if (labs(amount) > charwidth / 4L) {    /* } to make vi happy */
-#else
-    if (abs(amount) > charwidth / 4L) {
-#endif
+    if (labs(amount) > charwidth / 4L) {
         foo = 3*charwidth / 4;
         if (amount > 0)
             amount = ((amount+foo) / charwidth) * charwidth;
@@ -1001,18 +1017,18 @@ long snum(int size)
 
 void dounichar(long ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
 
     if (noligaturefi && 0xFB00<=ch && ch<=0xFB04) {
         switch (ch) {
-            case 0xFB00: strcpy(c,"ff");  break;
-            case 0xFB01: strcpy(c,"fi");  break;
-            case 0xFB02: strcpy(c,"fl");  break;
-            case 0xFB03: strcpy(c,"ffi"); break;
-            case 0xFB04: strcpy(c,"ffl"); break;
+            case 0xFB00: strcpy((char*)c,"ff");  break;
+            case 0xFB01: strcpy((char*)c,"fi");  break;
+            case 0xFB02: strcpy((char*)c,"fl");  break;
+            case 0xFB03: strcpy((char*)c,"ffi"); break;
+            case 0xFB04: strcpy((char*)c,"ffl"); break;
         }
         cc=c;
-        while (*cc) { outchar(*cc); cc++; }
+        while (*cc) { outchar((long)*cc); cc++; }
         return;
     }
     if (ch>0x7F)
@@ -1089,7 +1105,7 @@ void dochar(unsigned char ch)
 
 void symchar(unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     long ucs;
 
     ucs = oms_to_ucs[ch];
@@ -1158,7 +1174,7 @@ void symchar(unsigned char ch)
 
 void michar(unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     long ucs;
 
     if (allchar) {
@@ -1203,7 +1219,7 @@ void michar(unsigned char ch)
 
 void normchar(char flag, unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     const unsigned short *tex_to_ucs;
     long ucs;
 
@@ -1387,7 +1403,7 @@ void normchar(char flag, unsigned char ch)
 
 void t1char(unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     long ucs;
 
     if (allchar) {
@@ -1640,7 +1656,7 @@ void t1char(unsigned char ch)
 
 void ts1char(unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     long ucs;
 
     if (allchar) {
@@ -1775,7 +1791,7 @@ void ts1char(unsigned char ch)
 
 void t2char(char flag, unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     const unsigned short *tex_to_ucs;
     long ucs;
 
@@ -1809,7 +1825,7 @@ void t2char(char flag, unsigned char ch)
         case T2BFONT: tex_to_ucs=t2b_to_ucs; break;
         case T2CFONT: tex_to_ucs=t2c_to_ucs; break;
         case X2FONT : tex_to_ucs=x2_to_ucs;  break;
-        default : exit; /* not supported */
+        default : exit(41); /* not supported */
     }
     ucs = tex_to_ucs[ch];
     if (utf8) {
@@ -1906,7 +1922,7 @@ void t2char(char flag, unsigned char ch)
 
 void ot2char(unsigned char ch)
 {
-    unsigned char c[4] = {}, *cc;
+    unsigned char c[4] = {0}, *cc;
     long ucs;
 
     if (allchar) {
@@ -1989,11 +2005,7 @@ void outchar(long ch)
 
 /*     fprintf(stderr, "hor: %ld, ver: %ld\n", h, v); */
 
-#if defined(MSDOS) || defined(THINK_C)
     if (labs(v - currentline->vv) > lineheight / 2L)
-#else
-    if (abs(v - currentline->vv) > lineheight / 2L)
-#endif
         currentline = findline();
 
 #if 0
@@ -2294,7 +2306,7 @@ void fontdef(int x)
     if (fnt == NULL) {
         if ((fnt = (font *) malloc(sizeof(font))) == NULL) {
             perror("fontdef");
-            exit(1);
+            exit(40);
         }
         fnt->num = fntnum;
         new = 1;
@@ -2303,7 +2315,7 @@ void fontdef(int x)
         free(fnt->name);    /* free old name */
     if ((name = (char *) malloc((namelen+1) * sizeof(char))) == NULL) {
         perror("fontdef");
-        exit(1);
+        exit(40);
     }
     
     for (i = 0; i < namelen; i++)
