@@ -20,7 +20,8 @@
 #undef fopen
 #undef popen
 #undef pclose
-#define fopen(file, fmode)  fsyscp_fopen(file, fmode)
+extern FILE *generic_fsyscp_fopen(const char *name, const char *mode);
+#define fopen(file, fmode)  generic_fsyscp_fopen(file, fmode)
 #define popen(pcmd, pmode)  fsyscp_popen(pcmd, pmode)
 #define pclose(pstream) _pclose(pstream)
 #endif
@@ -157,6 +158,14 @@ copyfile_general(const char *s, struct header_list *cur_header)
  *   or figure files to be installed in the .../ps directory.
  */
       f = search(figpath, s, READBIN);
+#if defined(WIN32)
+      if (f == 0 && file_system_codepage != win32_codepage) {
+         int tmpcp = file_system_codepage;
+         file_system_codepage = win32_codepage;
+         f = search(figpath, s, READBIN);
+         file_system_codepage = tmpcp;
+      }
+#endif
       if (f == 0)
          f = search(headerpath, s, READBIN);
 #if defined(VMCMS) || defined (MVSXA)
@@ -168,19 +177,19 @@ copyfile_general(const char *s, struct header_list *cur_header)
       if (secure == 2) {
          strcat(errbuf, "\nNote that an absolute path or a relative path with .. are denied in -R2 mode.");
       }
-#else /* VMCMS */
+#else /* ! VMCMS */
       sprintf(errbuf,
     "Couldn't find figure file %s with MVS name %s; continuing.", s, trunc_s);
       if (secure == 2) {
          strcat(errbuf, "\nNote that an absolute path or a relative path with .. are denied in -R2 mode.");
       }
-#endif /* VMCMS */
-#else /* VMCMS || MVSXA */
+#endif /* ! VMCMS */
+#else /*  ! (VMCMS || MVSXA) */
       sprintf(errbuf, "Could not find figure file %.500s; continuing.", s);
       if (secure == 2) {
          strcat(errbuf, "\nNote that an absolute path or a relative path with .. are denied in -R2 mode.");
       }
-#endif /* VMCMS || MVSXA */
+#endif /* !(VMCMS || MVSXA) */
       break;
 #ifndef VMCMS
 #ifndef MVSXA
@@ -228,9 +237,10 @@ copyfile_general(const char *s, struct header_list *cur_header)
       }
       break;
    }
-   if (f==NULL)
+   if (f==NULL) {
+      found_problems = 1; /* continue, but eventually exit unsuccessfully */
       error(errbuf);
-   else {
+   } else {
       if (! quiet) {
 #if defined(VMCMS) || defined (MVSXA)
          if (strlen(s) + prettycolumn > STDOUTSIZE) {
@@ -243,7 +253,7 @@ copyfile_general(const char *s, struct header_list *cur_header)
 #if defined(VMCMS) || defined (MVSXA)
          fprintf(stderr, "<%s>", trunc_s);
 #else
-         fprintf(stderr, "<%s>", realnameoffile);
+         fprintf_str(stderr, "<%s>", realnameoffile);
 #endif
          fflush(stderr);
 #if defined(VMCMS) || defined (MVSXA)
@@ -867,6 +877,17 @@ cmdout(const char *s)
    lastspecial = 0;
 }
 
+void psnameout(const char *s) {
+   // we lead with a special, so we don't need the space.
+   lastspecial = 1 ;
+   cmdout(s) ;
+}
+
+void pslineout(const char *s) {
+   fputs(s, bitfile) ;
+   fprintf(bitfile, "\n");
+   linepos = 0;
+}
 
 static void
 chrcmd(char c)
@@ -1416,8 +1437,10 @@ initprinter(sectiontype *sect)
       if (tryepsf && isepsf == 0)
          error("We tried, but couldn't make it EPSF.");
       fprintf(bitfile, "%%%%Creator: %s\n", banner + 8);
-      if (*iname)
-         fprintf(bitfile, "%%%%Title: %s\n", iname);
+      if (*titlename)
+         fprintf(bitfile, "%%%%Title: %s\n", titlename);
+      else if (*iname)
+         fprintf(bitfile, "%%%%Title: %s\n", xbasename(iname));
 #ifdef CREATIONDATE
       jobtime = get_unique_time_if_given();
       if (jobtime == INVALID_EPOCH_VALUE) {
@@ -1448,6 +1471,25 @@ initprinter(sectiontype *sect)
       tell_needed_fonts();
       paperspec(finpapsiz->specdat, 1);
       fprintf(bitfile, "%%%%EndComments\n");
+/*
+ *   If we encode Type 3 fonts with an encoding vector, this can cause
+ *   Distiller's autoorientation to get confused.  We remedy this by
+ *   emitting underdocumented ViewingOrientation comments right after
+ *   EndComments.  Known defect: if a user "flips" the landscape to be
+ *   180 degrees using one of the \special{} commands available, the
+ *   document will be rendered in the viewer upside down.  (But only
+ *   with bitmap font encoding enabled and bitmapped fonts actually used.)
+ *   --tgr, 29 February 2020.
+ */
+      if (encodetype3 && bitmapfontseen) {
+         fprintf(bitfile, "%%%%BeginDefaults\n") ;
+         if (landscape) {
+            fprintf(bitfile, "%%%%ViewingOrientation: 0 -1 1 0\n") ;
+         } else {
+            fprintf(bitfile, "%%%%ViewingOrientation: 1 0 0 1\n") ;
+         }
+         fprintf(bitfile, "%%%%EndDefaults\n") ;
+      }
    }
    {
       int i, len;
