@@ -2,7 +2,7 @@
 ** BasicDVIReader.hpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -51,8 +51,8 @@ class BasicDVIReader : public StreamReader {
 
 	public:
 		explicit BasicDVIReader (std::istream &is);
-		virtual ~BasicDVIReader () =default;
 		virtual void executeAllPages ();
+		virtual void executeFontDefs ();
 		virtual double getXPos () const      {return 0;}
 		virtual double getYPos () const      {return 0;}
 		virtual void finishLine ()           {}
@@ -67,7 +67,11 @@ class BasicDVIReader : public StreamReader {
 		DVIVersion getDVIVersion () const {return _dviVersion;}
 		virtual int evalCommand (CommandHandler &handler, int &param);
 		virtual int executeCommand ();
+		void executePreamble ();
+		void executePostamble ();
 		void executePostPost ();
+		void goToPostamble ();
+		std::vector<uint32_t> collectBopOffsets ();
 		bool evalXDVOpcode (int op, CommandHandler &handler) const;
 
 		// The following template methods represent the single DVI commands. They
