@@ -2,7 +2,7 @@
 ** PSInterpreter.cpp                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -256,49 +256,50 @@ void PSInterpreter::callActions (InputReader &in) {
 		void (PSActions::*handler)(vector<double> &p);  // operation handler
 	};
 	static const unordered_map<string, Operator> operators {
-		{"applyscalevals", { 3, &PSActions::applyscalevals}},
-		{"clip",           { 0, &PSActions::clip}},
-		{"clippath",       { 0, &PSActions::clippath}},
-		{"closepath",      { 0, &PSActions::closepath}},
-		{"curveto",        { 6, &PSActions::curveto}},
-		{"eoclip",         { 0, &PSActions::eoclip}},
-		{"eofill",         { 0, &PSActions::eofill}},
-		{"fill",           { 0, &PSActions::fill}},
-		{"grestore",       { 0, &PSActions::grestore}},
-		{"grestoreall",    { 0, &PSActions::grestoreall}},
-		{"gsave",          { 0, &PSActions::gsave}},
-		{"image",          { 3, &PSActions::image}},
-		{"initclip",       { 0, &PSActions::initclip}},
-		{"lineto",         { 2, &PSActions::lineto}},
-		{"makepattern",    {-1, &PSActions::makepattern}},
-		{"moveto",         { 2, &PSActions::moveto}},
-		{"newpath",        { 1, &PSActions::newpath}},
-		{"querypos",       { 2, &PSActions::querypos}},
-		{"raw",            {-1, nullptr}},
-		{"restore",        { 1, &PSActions::restore}},
-		{"rotate",         { 1, &PSActions::rotate}},
-		{"save",           { 1, &PSActions::save}},
-		{"scale",          { 2, &PSActions::scale}},
-		{"setblendmode",   { 1, &PSActions::setblendmode}},
-		{"setcolorspace",  { 1, &PSActions::setcolorspace}},
-		{"setcmykcolor",   { 4, &PSActions::setcmykcolor}},
-		{"setdash",        {-1, &PSActions::setdash}},
-		{"setgray",        { 1, &PSActions::setgray}},
-		{"sethsbcolor",    { 3, &PSActions::sethsbcolor}},
-		{"setlinecap",     { 1, &PSActions::setlinecap}},
-		{"setlinejoin",    { 1, &PSActions::setlinejoin}},
-		{"setlinewidth",   { 1, &PSActions::setlinewidth}},
-		{"setmatrix",      { 6, &PSActions::setmatrix}},
-		{"setmiterlimit",  { 1, &PSActions::setmiterlimit}},
-		{"setnulldevice",  { 1, &PSActions::setnulldevice}},
-		{"setopacityalpha",{ 1, &PSActions::setopacityalpha}},
-		{"setshapealpha",  { 1, &PSActions::setshapealpha}},
-		{"setpagedevice",  { 0, &PSActions::setpagedevice}},
-		{"setpattern",     {-1, &PSActions::setpattern}},
-		{"setrgbcolor",    { 3, &PSActions::setrgbcolor}},
-		{"shfill",         {-1, &PSActions::shfill}},
-		{"stroke",         { 0, &PSActions::stroke}},
-		{"translate",      { 2, &PSActions::translate}},
+		{"applyscalevals",         { 3, &PSActions::applyscalevals}},
+		{"clip",                   { 0, &PSActions::clip}},
+		{"clippath",               { 0, &PSActions::clippath}},
+		{"closepath",              { 0, &PSActions::closepath}},
+		{"curveto",                { 6, &PSActions::curveto}},
+		{"eoclip",                 { 0, &PSActions::eoclip}},
+		{"eofill",                 { 0, &PSActions::eofill}},
+		{"fill",                   { 0, &PSActions::fill}},
+		{"grestore",               { 0, &PSActions::grestore}},
+		{"grestoreall",            { 0, &PSActions::grestoreall}},
+		{"gsave",                  { 0, &PSActions::gsave}},
+		{"image",                  { 3, &PSActions::image}},
+		{"initclip",               { 0, &PSActions::initclip}},
+		{"lineto",                 { 2, &PSActions::lineto}},
+		{"makepattern",            {-1, &PSActions::makepattern}},
+		{"moveto",                 { 2, &PSActions::moveto}},
+		{"newpath",                { 1, &PSActions::newpath}},
+		{"querypos",               { 2, &PSActions::querypos}},
+		{"raw",                    {-1, nullptr}},
+		{"restore",                { 1, &PSActions::restore}},
+		{"rotate",                 { 1, &PSActions::rotate}},
+		{"save",                   { 1, &PSActions::save}},
+		{"scale",                  { 2, &PSActions::scale}},
+		{"setblendmode",           { 1, &PSActions::setblendmode}},
+		{"setcolorspace",          { 1, &PSActions::setcolorspace}},
+		{"setcmykcolor",           { 4, &PSActions::setcmykcolor}},
+		{"setdash",                {-1, &PSActions::setdash}},
+		{"setfillconstantalpha",   { 1, &PSActions::setfillconstantalpha}},
+		{"setgray",                { 1, &PSActions::setgray}},
+		{"sethsbcolor",            { 3, &PSActions::sethsbcolor}},
+		{"setisshapealpha",        { 1, &PSActions::setisshapealpha}},
+		{"setlinecap",             { 1, &PSActions::setlinecap}},
+		{"setlinejoin",            { 1, &PSActions::setlinejoin}},
+		{"setlinewidth",           { 1, &PSActions::setlinewidth}},
+		{"setmatrix",              { 6, &PSActions::setmatrix}},
+		{"setmiterlimit",          { 1, &PSActions::setmiterlimit}},
+		{"setnulldevice",          { 1, &PSActions::setnulldevice}},
+		{"setpagedevice",          { 0, &PSActions::setpagedevice}},
+		{"setpattern",             {-1, &PSActions::setpattern}},
+		{"setrgbcolor",            { 3, &PSActions::setrgbcolor}},
+		{"setstrokeconstantalpha", { 1, &PSActions::setstrokeconstantalpha}},
+		{"shfill",                 {-1, &PSActions::shfill}},
+		{"stroke",                 { 0, &PSActions::stroke}},
+		{"translate",              { 2, &PSActions::translate}},
 	};
 	if (_actions) {
 		in.skipSpace();
