@@ -2,7 +2,7 @@
 ** SVGCharHandler.cpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,7 +25,7 @@
 using namespace std;
 
 
-void SVGCharHandler::setInitialContextNode (XMLElementNode *node) {
+void SVGCharHandler::setInitialContextNode (XMLElement *node) {
 	resetContextNode();
 	_initialContextNode = node;
 }
@@ -34,9 +34,9 @@ void SVGCharHandler::setInitialContextNode (XMLElementNode *node) {
 /** Changes the context element. All following nodes will be appended to this node.
  *  @param[in] node the new context node
  *  @return bare pointer to the new context node or 0 if context hasn't changed */
-XMLElementNode* SVGCharHandler::pushContextNode (unique_ptr<XMLElementNode> &&node) {
+XMLElement* SVGCharHandler::pushContextNode (unique_ptr<XMLElement> node) {
 	if (node && (_contextNodeStack.empty() || node.get() != _contextNodeStack.top())) {
-		XMLElementNode *nodeptr = node.get();
+		XMLElement *nodeptr = node.get();
 		contextNode()->append(std::move(node));
 		_contextNodeStack.push(nodeptr);
 		return nodeptr;
@@ -61,11 +61,11 @@ void SVGCharHandler::resetContextNode () {
 /** Creates and returns a new SVG text element.
  *  @param[in] x current x coordinate
  *  @param[in] y current y coordinate */
-unique_ptr<XMLElementNode> SVGCharTextHandler::createTextNode (double x, double y) const {
+unique_ptr<XMLElement> SVGCharTextHandler::createTextNode (double x, double y) const {
 	const Font *font = _font.get();
 	if (!font)
 		return nullptr;
-	auto textNode = util::make_unique<XMLElementNode>("text");
+	auto textNode = util::make_unique<XMLElement>("text");
 	if (_selectFontByClass)
 		textNode->addAttribute("class", string("f")+XMLString(_fontnum));
 	else {
@@ -77,7 +77,7 @@ unique_ptr<XMLElementNode> SVGCharTextHandler::createTextNode (double x, double
 	if (_vertical) {
 		textNode->addAttribute("writing-mode", "tb");
 		// align glyphs designed for horizontal layout properly
-		if (const PhysicalFont *pf = dynamic_cast<const PhysicalFont*>(font)) {
+		if (auto pf = dynamic_cast<const PhysicalFont*>(font)) {
 			if (!pf->getMetrics()->verticalLayout()) { // alphabetic text designed for horizontal layout?
 				x += pf->scaledAscent()/2.5; // move vertical baseline to the right by strikethrough offset
 				textNode->addAttribute("glyph-orientation-vertical", 90); // ensure rotation
@@ -87,6 +87,6 @@ unique_ptr<XMLElementNode> SVGCharTextHandler::createTextNode (double x, double
 	textNode->addAttribute("x", x);
 	textNode->addAttribute("y", y);
 	if (!_matrix.get().isIdentity())
-		textNode->addAttribute("transform", _matrix.get().getSVG());
+		textNode->addAttribute("transform", _matrix.get().toSVG());
 	return textNode;
 }
