@@ -2,7 +2,7 @@
 ** Unicode.hpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -23,11 +23,13 @@
 
 #include <string>
 
-struct Unicode
-{
+struct Unicode {
 	static bool isValidCodepoint (uint32_t code);
 	static uint32_t charToCodepoint (uint32_t c);
 	static std::string utf8 (int32_t c);
+	static uint32_t fromSurrogate (uint32_t high, uint32_t low);
+	static uint32_t fromSurrogate (uint32_t cp);
+	static uint32_t toSurrogate (uint32_t cp);
 	static int32_t aglNameToCodepoint (const std::string &name);
 };
 
