@@ -2,7 +2,7 @@
 ** CalculatorTest.cpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,7 +25,10 @@
 
 TEST(CalculatorTest, eval) {
 	Calculator calc;
+	EXPECT_EQ(calc.eval("2"),             2);
+	EXPECT_EQ(calc.eval("-2"),           -2);
 	EXPECT_EQ(calc.eval("2+3+4"),         9);
+	EXPECT_EQ(calc.eval(" 2 + 3+ 4 "),    9);
 	EXPECT_EQ(calc.eval("2*3+4"),        10);
 	EXPECT_EQ(calc.eval("2+3*4"),        14);
 	EXPECT_EQ(calc.eval("(2+3)*4"),      20);
@@ -36,7 +39,11 @@ TEST(CalculatorTest, eval) {
 	EXPECT_EQ(calc.eval("(1+2)-(3+4)"),  -4);
 	EXPECT_EQ(calc.eval("-2+3+4"),        5);
 	EXPECT_EQ(calc.eval("3/2"),         1.5);
+	EXPECT_EQ(calc.eval("1+3/2"),       2.5);
+	EXPECT_EQ(calc.eval("3/2+1"),       2.5);
 	EXPECT_EQ(calc.eval("3%2"),           1);
+	EXPECT_EQ(calc.eval("1+3%2"),         2);
+	EXPECT_EQ(calc.eval("3%2+1"),         2);
 	EXPECT_EQ(calc.eval("-(4)"),         -4);
 }
 
@@ -51,6 +58,8 @@ TEST(CalculatorTest, variables) {
 
 	calc.setVariable("b", 3);
 	EXPECT_EQ(calc.eval("a+b"), 5);
+	EXPECT_EQ(calc.eval("a+a*b"), 8);
+	EXPECT_EQ(calc.eval("a*a+b"), 7);
 	EXPECT_EQ(calc.eval("2a+2b"), 10);
 	EXPECT_EQ(calc.eval("2a(1+2b)"), 28);
 }
@@ -64,6 +73,8 @@ TEST(CalculatorTest, exceptions) {
 	ASSERT_THROW(calc.eval("1%0"), CalculatorException);
 	ASSERT_THROW(calc.eval("2*(3+4"), CalculatorException);
 	ASSERT_THROW(calc.eval("2*(3+4))"), CalculatorException);
+	ASSERT_THROW(calc.eval("2*3+4)"), CalculatorException);
+	ASSERT_THROW(calc.eval("2(3+4"), CalculatorException);
 }
 
 
