@@ -121,11 +121,12 @@ void indwrite(char *filename, struct index *ind, int pagenum)
 {
 	int i,j,hpoint=0;
 	char datama[2048],lbuff[BUFFERLEN];
-	FILE *fp;
+	FILE *fp=NULL;
 	int conv_euc_to_euc;
 
-	if (filename && kpse_out_name_ok(filename)) fp=fopen(filename,"wb");
-	else {
+	if (filename && kpse_out_name_ok(filename))
+		fp=fopen(filename,"wb");
+	if (fp == NULL) {
 		fp=stdout;
 #ifdef WIN32
 		setmode(fileno(fp), _O_BINARY);
@@ -218,7 +219,15 @@ void indwrite(char *filename, struct index *ind, int pagenum)
 				if ((alphabet(ind[i-1].dic[0][0]))||(japanese(ind[i-1].dic[0]))){
 					fputs(group_skip,fp);
 					if (lethead_flag && symbol_flag) {
-						fprintf(fp,"%s%s%s",lethead_prefix,symbol,lethead_suffix);
+						if (strlen(symbol)) {
+							fprintf(fp,"%s%s%s",lethead_prefix,symbol,lethead_suffix);
+						}
+						else if (lethead_flag>0) {
+							fprintf(fp,"%s%s%s",lethead_prefix,symhead_positive,lethead_suffix);
+						}
+						else if (lethead_flag<0) {
+							fprintf(fp,"%s%s%s",lethead_prefix,symhead_negative,lethead_suffix);
+						}
 					}
 				}
 			}
@@ -309,7 +318,7 @@ void indwrite(char *filename, struct index *ind, int pagenum)
 	}
 	fputs(postamble,fp);
 
-	if (filename) fclose(fp);
+	if (fp!=stdout) fclose(fp);
 }
 
 /*   write page block   */
@@ -326,7 +335,11 @@ static void printpage(struct index *ind, FILE *fp, int num, char *lbuff)
 	for(j=0;j<ind[num].num;j++) {
 		cc=range_check(ind[num],j,lbuff);
 		if (cc>j) {
-			if (pnumconv(ind[num].p[j].page,ind[num].p[j].attr[0])==pnumconv(ind[num].p[cc].page,ind[num].p[cc].attr[0])) {
+			int epage = pnumconv(ind[num].p[cc].page,
+    		       	                     ind[num].p[cc].attr[0]);
+			int bpage = pnumconv(ind[num].p[j].page,
+			                     ind[num].p[j].attr[0]);
+			if (epage==bpage) {
 				j=cc-1;
 				continue;
 			}
@@ -337,20 +350,18 @@ static void printpage(struct index *ind, FILE *fp, int num, char *lbuff)
 			if (strlen(ind[num].p[j].enc)>0) {
 				SPRINTF(buff,"%s%s%s",encap_prefix,ind[num].p[j].enc,encap_infix);
 			}
-			if (strlen(suffix_3p)>0 && (pnumconv(ind[num].p[cc].page,ind[num].p[cc].attr[0])-pnumconv(ind[num].p[j].page,ind[num].p[j].attr[0]))==2) {
-				SAPPENDF(buff,"%s",ind[num].p[j].page);
+			/* print beginning of range */
+			SAPPENDF(buff,"%s",ind[num].p[j].page);
+			if (strlen(suffix_3p)>0 && (epage-bpage)==2) {
 				SAPPENDF(buff,"%s",suffix_3p);
 			}
-			else if (strlen(suffix_mp)>0 && (pnumconv(ind[num].p[cc].page,ind[num].p[cc].attr[0])-pnumconv(ind[num].p[j].page,ind[num].p[j].attr[0]))>=2) {
-				SAPPENDF(buff,"%s",ind[num].p[j].page);
+			else if (strlen(suffix_mp)>0 && (epage-bpage)>=2) {
 				SAPPENDF(buff,"%s",suffix_mp);
 			}
-			else if (strlen(suffix_2p)>0 && (pnumconv(ind[num].p[cc].page,ind[num].p[cc].attr[0])-pnumconv(ind[num].p[j].page,ind[num].p[j].attr[0]))==1) {
-				SAPPENDF(buff,"%s",ind[num].p[j].page);
+			else if (strlen(suffix_2p)>0 && (epage-bpage)==1) {
 				SAPPENDF(buff,"%s",suffix_2p);
 			}
 			else {
-				SAPPENDF(buff,"%s",ind[num].p[j].page);
 				SAPPENDF(buff,"%s",delim_r);
 				SAPPENDF(buff,"%s",ind[num].p[cc].page);
 			}
