@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: epstopdf.pl 41577 2016-06-30 16:38:01Z karl $
+# $Id: epstopdf.pl 48681 2018-09-16 23:03:58Z karl $
 # (Copyright lines below.)
 #
 # Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,13 @@
 #
 # emacs-page
 #
-my $ver = "2.26";
+my $ver = "2.28";
+#  2018/09/17 v2.28 (Karl Berry)
+#    * -dCompatibilityLevel=1.5 by default, since gs9.25 switched to 1.7.
+#  2017/09/14 v2.27 (Karl Berry)
+#    * extract value from --gsopt with $3 not $2 (extra regexp group
+#      added previously), and check it with ^(...)$ so anchors apply to all.
+#      (report to Karl from Yannick Berker, 7 Sep 2017 14:07:09.)
 #  2017/01/07 v2.26 (Norbert Preining, Karl Berry)
 #    * allow cmdline of infile outfile.pdf.
 #    * explicitly allow -o as abbreviation for --outfile,
@@ -183,9 +189,9 @@ my $ver = "2.26";
 ### emacs-page
 ### program identification
 my $program = "epstopdf";
-my $ident = '($Id: epstopdf.pl 41577 2016-06-30 16:38:01Z karl $)' . " $ver";
+my $ident = '($Id: epstopdf.pl 48681 2018-09-16 23:03:58Z karl $)' . " $ver";
 my $copyright = <<END_COPYRIGHT ;
-Copyright 2009-2017 Karl Berry et al.
+Copyright 2009-2018 Karl Berry et al.
 Copyright 2002-2009 Gerben Wierda et al.
 Copyright 1998-2001 Sebastian Rahtz et al.
 License RBSD: Revised BSD <http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5>
@@ -295,6 +301,7 @@ my %optcheck = qw<
   MonoImageFilter /(CCITTFaxEncode|FlateEncode|RunLengthEncode)
   MonoImageResolution \d+
   NOCIE true
+  NODISPLAY true
   NOEPS true
   NOINTERPOLATE true
   NOPSICC true
@@ -343,6 +350,9 @@ replaced by ".pdf".  An output name ending with .pdf can also be given
 as a second argument on the command line, or the --outfile (-o) option
 can be used with any name.
 
+The output is PDF 1.5 by default; use --gsopt=-dCompatibilityLevel=1.7
+(for example) to change this.
+
 The resulting output is guaranteed to start at the 0,0 coordinate, and
 sets a page size exactly corresponding to the BoundingBox.  Thus, the
 result does not need any cropping, and the PDF MediaBox is correct.
@@ -545,6 +555,7 @@ push @GS, '-q' if $::opt_quiet;
 push @GS, $::opt_safer ? '-dSAFER' : '-dNOSAFER';
 push @GS, '-dNOPAUSE';
 push @GS, '-dBATCH';
+push @GS, '-dCompatibilityLevel=1.5';
 
 ### option device
 if ($::opt_device) {
@@ -628,9 +639,9 @@ foreach my $gsopt (@::opt_gsopt) {
     my $ok = 0;
     if ($gsopt =~ /^-[dD]([A-Za-z0-9]+)(=(.*))?$/) {
       my $name = $1;
-      my $value = $2;
+      my $value = $3;
       $value = 'true' if not defined $value;
-      if ($optcheck{$name} and $value =~ /^$optcheck{$name}$/) {
+      if ($optcheck{$name} and $value =~ /^($optcheck{$name})$/) {
         $ok = 1;
       }
       else {
