@@ -61,6 +61,9 @@ const_string CTANGLEHELP[] = {
     "+p          print progress report messages",
     "+/-q        shortcut for '-bhp'; also '--quiet' (default)",
     "+/-v        shortcut for '+bhp'; also '--verbose'",
+    "-c          ignore temporary output irrespective of changes",
+    "-dN         set 'kpathsea_debug' to N (0..127)",
+    "+k          keep separators in numeric literals in the output",
     "+s          print usage statistics",
     "--help      display this help and exit",
     "--version   output version information and exit",
@@ -80,6 +83,8 @@ const_string CWEAVEHELP[] = {
     "+p          print progress report messages",
     "+/-q        shortcut for '-bhp'; also '--quiet' (default)",
     "+/-v        shortcut for '+bhp'; also '--verbose'",
+    "-c          ignore temporary output irrespective of changes",
+    "-dN         set 'kpathsea_debug' to N (0..127)",
     "-e          do not enclose C material in \\PB{...}",
     "-f          do not force a newline after every C statement in output",
     "-i          suppress indentation of parameter declarations",
@@ -87,6 +92,7 @@ const_string CWEAVEHELP[] = {
     "-x          omit indices, section names, table of contents",
     "+lX         use macros for language X as of Xcwebmac.tex",
     "+s          print usage statistics",
+    "+t          treat 'typename' in a template like 'typedef'",
     "--help      display this help and exit",
     "--version   output version information and exit",
     NULL
@@ -105,6 +111,8 @@ const_string CTWILLHELP[] = {
     "+p          print progress report messages",
     "+/-q        shortcut for '-bhp'; also '--quiet' (default)",
     "+/-v        shortcut for '+bhp'; also '--verbose'",
+    "-c          ignore temporary output irrespective of changes",
+    "-dN         set 'kpathsea_debug' to N (0..127)",
     "-e          do not enclose C material in \\PB{...}",
     "-f          do not force a newline after every C statement in output",
     "-i          suppress indentation of parameter declarations",
@@ -113,6 +121,7 @@ const_string CTWILLHELP[] = {
     "+P          \\input ctproofmac.tex instead of ctwimac.tex",
     "+/-lX       use macros for language X as of Xct{wi|proof}mac.tex",
     "+s          print usage statistics",
+    "+t          treat 'typename' in a template like 'typedef'",
     "--help      display this help and exit",
     "--version   output version information and exit",
     NULL
