@@ -264,6 +264,7 @@ inline static int run_put_next(lua_State * L)
                             token_link(t) = x;
                         }
                         t = x;
+                        lua_pop(L, 1);
                     }
                     lua_pop(L, 1);
                 }
@@ -613,23 +614,35 @@ static int run_scan_argument(lua_State * L) /* HH */
         get_token();
     } while ((cur_cmd == spacer_cmd) || (cur_cmd == relax_cmd));
     if (cur_cmd == left_brace_cmd) {
+        int exp = 1;
+        if (lua_type(L, 1) == LUA_TBOOLEAN) {
+            exp = lua_toboolean(L, 1);
+        }
         back_input();
         saved_defref = def_ref;
-        (void) scan_toks(false, true);
+        (void) scan_toks(false, exp);
         t = def_ref;
         def_ref = saved_defref;
         tokenlist_to_luastring(L,t);
         flush_list(t);
     } else if (cur_cmd == call_cmd) {
         halfword saved_cur_tok = cur_tok;
+        int exp = 1;
+        if (lua_type(L, 1) == LUA_TBOOLEAN) {
+            exp = lua_toboolean(L, 1);
+        }
         cur_tok = right_brace_token + '}';
         back_input();
-        cur_tok = saved_cur_tok;
-        back_input();
+        if (exp) {
+            cur_tok = saved_cur_tok;
+            back_input();
+        } else {
+            expand();
+        }
         cur_tok = left_brace_token + '{';
         back_input();
         saved_defref = def_ref;
-        (void) scan_toks(false, true);
+        (void) scan_toks(false, exp);
         t = def_ref;
         def_ref = saved_defref;
         tokenlist_to_luastring(L,t);
@@ -739,6 +752,7 @@ static int run_lookup(lua_State * L)
     return 1;
 }
 
+/*
 static int lua_tokenlib_is_defined(lua_State * L)
 {
     const char *s;
@@ -753,6 +767,26 @@ static int lua_tokenlib_is_defined(lua_State * L)
     lua_pushnil(L);
     return 1;
 }
+*/
+
+static int lua_tokenlib_is_defined(lua_State * L)
+{
+    int b = 0;
+    if (lua_type(L, 1) == LUA_TSTRING) {
+        size_t l;
+        const char *s = lua_tolstring(L, 1, &l);
+        if (l > 0) {
+            int cs = string_lookup(s, l);
+            if (lua_toboolean(L, 2)) {
+                b = cs != undefined_control_sequence;
+            } else {
+                b = eq_type(cs) != undefined_cs_cmd;
+            }
+        }
+    }
+    lua_pushboolean(L, b);
+    return 1;
+}
 
 static int run_build(lua_State * L)
 {
@@ -812,34 +846,40 @@ inline static int lua_tokenlib_get_command(lua_State * L)
 
 inline static int lua_tokenlib_get_index(lua_State * L)
 {
+    int cmd, chr;
     lua_token *n = check_istoken(L, 1);
     halfword t = token_info(n->token);
-    int cmd = (t >= cs_token_flag ? eq_type(t - cs_token_flag) : token_cmd(t));
-    halfword e = equiv(t - cs_token_flag);
+    if (t >= cs_token_flag) {
+        cmd = eq_type(t - cs_token_flag);
+        chr = equiv(t - cs_token_flag);
+    } else {
+        cmd = token_cmd(t);
+        chr = token_chr(t);
+    }
     switch (cmd) {
         case assign_int_cmd:
-            e -= count_base;
+            chr -= count_base;
             break;
         case assign_attr_cmd:
-            e -= attribute_base;
+            chr -= attribute_base;
             break;
         case assign_dimen_cmd:
-            e -= dimen_base;
+            chr -= scaled_base;
             break;
         case assign_glue_cmd:
-            e -= skip_base;
+            chr -= skip_base;
             break;
         case assign_mu_glue_cmd:
-            e -= mu_skip_base;
+            chr -= mu_skip_base;
             break;
         case assign_toks_cmd:
-            e -= toks_base;
+            chr -= toks_base;
             break;
         default:
             break;
     }
-    if ((e >= 0) && (e <= 65535)) {
-        lua_pushinteger(L, e);
+    if (chr >= 0 && chr <= 65535) {
+        lua_pushinteger(L, chr);
     } else {
         lua_pushnil(L);
     }
@@ -1158,6 +1198,9 @@ static int set_macro(lua_State * L)
         if (n == 1)
             return 0;
         ct = (int) lua_tointeger(L, 1);
+        if (!valid_catcode_table(ct)) {
+            ct = cat_code_table_par;
+        }
         name = lua_tolstring(L, 2, &lname);
         if (n > 2)
             str = lua_tolstring(L, 3, &lstr);
@@ -1218,10 +1261,14 @@ static int set_macro(lua_State * L)
                         str += _s ;
                         break ;
                     } else {
+                        if (_lname == 0) {
+                            _lname = _lname + _s ;
+                            str += _s ;
+                        }
                         break ;
                     }
                 }
-                if (_s > 0) {
+                if (_lname > 0) {
                     /* we have a potential \cs */
                     _cs = string_lookup(_name, _lname);
                     if (_cs == undefined_control_sequence) {
