@@ -461,6 +461,7 @@ const char *math_param_names[] = {
     "stackvgap", "stacknumup", "stackdenomdown",
     "fractionrule", "fractionnumvgap", "fractionnumup",
     "fractiondenomvgap", "fractiondenomdown", "fractiondelsize",
+    "skewedfractionhgap", "skewedfractionvgap",
     "limitabovevgap", "limitabovebgap", "limitabovekern",
     "limitbelowvgap", "limitbelowbgap", "limitbelowkern",
     "nolimitsubfactor", "nolimitsupfactor", /* bonus */
@@ -720,7 +721,9 @@ void display_normal_noad(pointer p)
         }
         break;
     case radical_noad:
-        if (subtype(p) == 6)
+        if (subtype(p) == 7)
+            tprint_esc("Uhextensible");
+        else if (subtype(p) == 6)
             tprint_esc("Udelimiterover");
         else if (subtype(p) == 5)
             tprint_esc("Udelimiterunder");
@@ -1445,8 +1448,7 @@ void set_math_char(mathcodeval mval)
         } else {
             math_class_to_type(subtype(p),mval.class_value);
         }
-        vlink(tail) = p;
-        tail = p;
+        tail_append(p);
     }
 }
 
@@ -1471,8 +1473,7 @@ void math_char_in_text(mathcodeval mval)
         back_input();
     } else {
         p = new_char(fam_fnt(mval.family_value, text_size), mval.character_value);
-        vlink(tail) = p;
-        tail = p;
+        tail_append(p);
     }
 }
 
@@ -1576,11 +1577,17 @@ static void scan_delimiter(pointer p, int r)
     return;
 }
 
+/*tex
+    Because \LATEX\ expect some different defaults, we now have |math_defaults_mode_par| controlling
+    the style. By using a variable we can remain downward compatible.
+*/
+
 void math_radical(void)
 {
     halfword q;
     int chr_code = cur_chr;
     halfword options = 0;
+    halfword used_style = cramped_style(m_style);
     tail_append(new_node(radical_noad, chr_code));
     q = new_node(delim_node, 0);
     left_delimiter(tail) = q;
@@ -1599,32 +1606,42 @@ void math_radical(void)
         }
     }
     radicaloptions(tail) = options;
-    if (chr_code == 0)
+    if (chr_code == 0) {
         /*tex \.{\\radical} */
         scan_delimiter(left_delimiter(tail), tex_mathcode);
-    else if (chr_code == 1)
+    } else if (chr_code == 1) {
         /*tex \.{\\Uradical} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 2)
+    } else if (chr_code == 2) {
         /*tex \.{\\Uroot} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 3)
+    } else if (chr_code == 3) {
         /*tex \.{\\Uunderdelimiter} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 4)
+        if (math_defaults_mode_par > 0) {
+            used_style = sub_style(m_style);
+        }
+    } else if (chr_code == 4) {
         /*tex \.{\\Uoverdelimiter} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 5)
+        if (math_defaults_mode_par > 0) {
+            used_style = sup_style(m_style);
+        }
+    } else if (chr_code == 5) {
         /*tex \.{\\Udelimiterunder} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 6)
+        if (math_defaults_mode_par > 0) {
+            used_style = m_style;
+        }
+    } else if (chr_code == 6) {
         /*tex \.{\\Udelimiterover} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else if (chr_code == 7)
+    } else if (chr_code == 7) {
         /*tex \.{\\Uhextensible} */
         scan_delimiter(left_delimiter(tail), umath_mathcode);
-    else
+    } else {
         confusion("math_radical");
+    }
     if (chr_code == 7) {
         /*tex type will change */
         q = new_node(sub_box_node, 0);
@@ -1644,12 +1661,12 @@ void math_radical(void)
             vlink(degree(tail)) = null;
             q = new_node(math_char_node, 0);
             nucleus(tail) = q;
-            (void) scan_math(nucleus(tail), cramped_style(m_style));
+            (void) scan_math(nucleus(tail), used_style);
         }
     } else {
         q = new_node(math_char_node, 0);
         nucleus(tail) = q;
-        (void) scan_math(nucleus(tail), cramped_style(m_style));
+        (void) scan_math(nucleus(tail), used_style);
     }
 }
 
@@ -1925,6 +1942,8 @@ void math_fraction(void)
                 while (1) {
                     if (scan_keyword("exact")) {
                         options = options | noad_option_exact ;
+                    } else if (scan_keyword("norule")) {
+                        options = options | noad_option_no_rule ;
                     } else {
                         break;
                     }
@@ -2551,8 +2570,12 @@ void after_math(void)
     }
     if (mode == -m) {
         /*tex end of equation number */
+      RECHECK:
         if (cur_cmd == math_shift_cmd) {
             check_second_math_shift();
+        } else if (suppress_mathpar_error_par && cur_cmd == par_end_cmd) {
+            get_x_token();
+            goto RECHECK;
         } else {
             check_display_math_end();
         }
@@ -2618,7 +2641,8 @@ void after_math(void)
             tail_append(new_dir(math_direction_par));
         }
         run_mlist_to_hlist(p, (mode > 0), text_style);
-        vlink(tail) = vlink(temp_head);
+	try_couple_nodes(tail,vlink(temp_head));
+
         while (vlink(tail) != null) {
             tail = vlink(tail);
         }
@@ -2686,8 +2710,12 @@ void after_math(void)
 void finish_display_alignment(pointer p, pointer q, halfword saved_prevdepth)
 {
     do_assignments();
+  RECHECK:
     if (cur_cmd == math_shift_cmd) {
         check_second_math_shift();
+    } else if (suppress_mathpar_error_par && cur_cmd == par_end_cmd) {
+        get_x_token();
+        goto RECHECK;
     } else {
         check_display_math_end();
     }
