@@ -455,8 +455,13 @@ is_good_dvi_file(const char *filename, Boolean from_history)
 
     if ((ret = REALPATH(real_filename, canonical_path)) == NULL) {
 	/* REALPATH failed, use real_filename */
-	strncpy(canonical_path, real_filename, MAXPATHLEN);
-	canonical_path[MAXPATHLEN] = '\0';
+	size_t n = strlen(real_filename);
+	if (n >= MAXPATHLEN) {
+	  n = MAXPATHLEN - 1;
+	}
+
+	memcpy(canonical_path, real_filename, n);
+	canonical_path[n] = '\0';
 	ret = canonical_path;
     }
     free(real_filename);
@@ -641,12 +646,12 @@ init_check_resources(void)
 
     /* margins */
     if (resource.sidemargin)
-	resource.sidemargin_int = atopix(resource.sidemargin, False);
+	resource.sidemargin_int = atopix(resource.sidemargin);
     if (resource.topmargin)
-	resource.topmargin_int = atopix(resource.topmargin, False);
-    resource.xoffset_int = resource.xoffset ? atopix(resource.xoffset, True)
+	resource.topmargin_int = atopix(resource.topmargin);
+    resource.xoffset_int = resource.xoffset ? atopix_signed(resource.xoffset)
 	: resource.pixels_per_inch;
-    resource.yoffset_int = resource.yoffset ? atopix(resource.yoffset, True)
+    resource.yoffset_int = resource.yoffset ? atopix_signed(resource.yoffset)
 	: resource.pixels_per_inch;
 
     /* paper type */
